package kernel.ui;

import kernel.ps.ProcessManager;
import kernel.ps.ProcessManager.PID;
import haxe.ds.ReadOnlyArray;
import kernel.ui.TermWriteable;
import kernel.peripherals.Peripherals.Peripheral;

/**
	Responsable for creating window context, forwarding UI events to the context
	and switching context to real screens.
**/
class WindowManager {
	/**
		Depends on: KernelEvents, Peripheral
	**/
	private static var currentMainContext:WindowContext;

	private static final outputMap:Map<String, WindowContext> = new Map();

	@:allow(kernel.Init)
	private static function init() {
		KernelEvents.onKey.handle(params -> {
			if (currentMainContext != null) {
				if (currentMainContext.eventDelegate != null) {
					var foo = currentMainContext.eventDelegate.getEventHandlers();
					foo.onKey != null ? foo.onKey.invoke(params) : null;
				} else {
					currentMainContext.onKeyTrigger.trigger(params);
				}
			}
		});

		KernelEvents.onKeyUp.handle(keyCode -> {
			if (currentMainContext != null) {
				if (currentMainContext.eventDelegate != null) {
					var foo = currentMainContext.eventDelegate.getEventHandlers();
					foo.onKeyUp != null ? foo.onKeyUp.invoke(keyCode) : null;
				} else {
					currentMainContext.onKeyUpTrigger.trigger(keyCode);
				}
			}
		});

		KernelEvents.onMouseClick.handle(params -> {
			if (currentMainContext != null) {
				if (currentMainContext.eventDelegate != null) {
					var foo = currentMainContext.eventDelegate.getEventHandlers();
					foo.onClick != null ? foo.onClick.invoke(params) : null;
				} else {
					currentMainContext.onClickTrigger.trigger(params);
				}
			}
		});

		KernelEvents.onMouseDrag.handle(params -> {
			if (currentMainContext != null) {
				if (currentMainContext.eventDelegate != null) {
					var foo = currentMainContext.eventDelegate.getEventHandlers();
					foo.onMouseDrag != null ? foo.onMouseDrag.invoke(params) : null;
				} else {
					currentMainContext.onMouseDragTrigger.trigger(params);
				}
			}
		});

		KernelEvents.onMouseScroll.handle(params -> {
			if (currentMainContext != null) {
				if (currentMainContext.eventDelegate != null) {
					var foo = currentMainContext.eventDelegate.getEventHandlers();
					foo.onMouseScroll != null ? foo.onMouseScroll.invoke(params) : null;
				} else {
					currentMainContext.onMouseScrollTrigger.trigger(params);
				}
			}
		});

		KernelEvents.onMouseUp.handle(params -> {
			if (currentMainContext != null) {
				if (currentMainContext.eventDelegate != null) {
					var foo = currentMainContext.eventDelegate.getEventHandlers();
					foo.onMouseUp != null ? foo.onMouseUp.invoke(params) : null;
				} else {
					currentMainContext.onMouseUpTrigger.trigger(params);
				}
			}
		});

		KernelEvents.onPaste.handle(text -> {
			if (currentMainContext != null) {
				if (currentMainContext.eventDelegate != null) {
					var foo = currentMainContext.eventDelegate.getEventHandlers();
					foo.onPaste != null ? foo.onPaste.invoke(text) : null;
				} else {
					currentMainContext.onPasteTrigger.trigger(text);
				}
			}
		});

		KernelEvents.onMonitorTouch.handle(params -> {
			// TODO
		});

		KernelEvents.onChar.handle(char -> {
			if (currentMainContext != null) {
				if (currentMainContext.eventDelegate != null) {
					var foo = currentMainContext.eventDelegate.getEventHandlers();
					foo.onChar != null ? foo.onChar.invoke(char) : null;
				} else {
					currentMainContext.onCharTrigger.trigger(char);
				}
			}
		});
	}

	/**
		Same as createNewBufferedContext because it is the most simple context.
	**/
	public static function createNewContext():WindowContext {
		return createNewBufferedContext();
	}

	public static function createNewBufferedContext():WindowContext {
		var newContext = new WindowContext(new BufferedVirtualTermWriter());

		return newContext;
	}

	public static function createNewStatelessContext():{ctx:WindowContext, setRenderFunc:(() -> Void)->Void, requestRender:Void->Void} {
		var writer = new StatelessVirtualTermWriter();
		var newContext = new WindowContext(writer);

		return {ctx: newContext, setRenderFunc: writer.setRenderFunc, requestRender: writer.requestRender};
	}

	public static function getOutputs():ReadOnlyArray<String> {
		var arr = Peripheral.getAllScreens().map(screen -> return screen.getAddr());
		arr.push("main");
		return arr;
	}

	/**
		Move context to output. If output is "main", context will be moved to main screen.
	**/
	public static function focusContextToOutput(context:WindowContext, output:String) {
		var target:TermWriteable;
		if (output == "main") {
			target = MainTerm.instance;
			currentMainContext = context;
		} else {
			target = Peripheral.getScreen(output);

			if (target == null) {
				// output target not found
				return;
			}
		}

		if (outputMap.exists(output)) {
			outputMap[output].disable();
		}

		outputMap[output] = context;
		context.setTarget(target);
		context.enable();
	}

	public static function getContextByPID(pid:PID):ReadOnlyArray<WindowContext> {
		var handle = ProcessManager.getProcess(pid);
		if (handle == null) {
			return [];
		}

		return handle.getWindowContexts();
	}
}
